#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec  4 13:42:45 2019

@author: wolterse
"""
import os
import numpy as np
import gdal
import gdalnumeric
from netCDF4 import Dataset
import osr
import rasterio


def get_footprint_value(infile):
    footprint_value =  'MULTIPOLYGON (((-180 %latmax%, -180 %latmin%, 180 %latmin%, 180 %latmax%s, -180 %latmax%)))'
    with rasterio.open(infile) as dataset:
        # Read the dataset's valid data mask as a ndarray.
        mask = dataset.dataset_mask()
        ymax_array = mask.max(axis=1)
        firsthit = 0
        lasthit = 0
        for yindex in range(0, len(ymax_array)):
            if ymax_array[yindex] == 255:
                firsthit = yindex
                break

        for yindex in range(len(ymax_array)-1 ,0, -1):
            if ymax_array[yindex] == 255:
                lasthit = yindex
                break

        last = rasterio.transform.xy(dataset.transform, lasthit, 0)
        latmin = last[1]
        first = rasterio.transform.xy(dataset.transform, firsthit, 0)
        latmax = first[1]
        footprint_value = footprint_value.replace('%latmax%',str(latmax))
        footprint_value = footprint_value.replace('%latmin%',str(latmin))
        return footprint_value


def convert_nc2tiff(input_file, output_dir, tif_settings):
    latpixels2skip = 20
    version_major = tif_settings[0]['tif_file_name'][-7]
    version_minor = tif_settings[0]['tif_file_name'][-6]
    version_patch = tif_settings[0]['tif_file_name'][-5]
    tiftagdate = tif_settings[0]['tif_file_name'].split('_')[3]
    TIFTAGS= {"TIFFTAG_COPYRIGHT": 'VITO',
              "TIFFTAG_DATETIME" : "%s-%s-%s" % (tiftagdate[0:4],tiftagdate[4:6],tiftagdate[6:8]),
              "TIFFTAG_SOFTWARE" : "Workflow:%s.%s.%s" % (version_major, version_minor, version_patch)}
    for tif_setting in tif_settings:
        scale = tif_setting['scale']
        unit_conversion_factor = tif_setting['unit_conversion_factor']
        offset = tif_setting['offset']
        #update processingmode in description
        processingmode = tif_setting['processingmode']
        description = tif_setting['metadata']['description'].replace('%processing_mode%', processingmode)
        tif_setting['metadata']['description'] = description

        metadata = tif_setting['metadata']
        nodata =  tif_setting['nodata']
        if tif_setting['dtype'] == 'byte':
            dtype = gdal.GDT_Byte
        else:
            dtype = gdal.GDT_Int16
        band_name = tif_setting['nc_band_name']
        tif_name = tif_setting['tif_file_name'].replace('<band>',tif_setting['tif_band_name'])

        d_nc = Dataset(input_file, 'r', format="NETCDF4")

        lat_bounds = d_nc.variables['latitude_bounds'][:]
        lon_bounds = d_nc.variables['longitude_bounds'][:]
        lat = (lat_bounds[0, :, 0] + lat_bounds[0, :, 1]) / 2.
        lon = (lon_bounds[0, :, 0] + lon_bounds[0, :, 1]) / 2.

        # test on variable CO
        d_var = np.flipud(d_nc.variables[band_name][0, :, :])

        #testarrau
        #d_test=np.array([ -1, 0 ,1 ,np.nan])   #test_array
        #d_var = np.ma.masked_array(d_test, mask=[0, 0, 0, 1])


        # set values <0 to nodata, currently disabled, so negative values will be in the resutling image
        #np.ma.masked_less(d_var, 0, copy=False)


        #scaling
        # PV = DV * input_scale + PV_offset
        # DV = (PV - PV_offset)/ input_scale
        d_var = d_var / unit_conversion_factor          #e.g from mol to µmol
        d_var_scaled = (d_var - offset)/scale
        d_var_scaled = d_var_scaled.astype('int16')

        #setting masked data with nodata value
        d_var_scaled_masked = d_var_scaled.filled(nodata)

        # #getting the footprint
        # i_val = np.where(np.isfinite(d_var))
        # min_lat = round(lat[i_val[0]][0])
        # max_lat = round(lat[i_val[0]][-1])
        # footprint_bbox = -180, min_lat, 180, max_lat

        # generate the geotransform
        nx = len(lon)
        ny = len(lat)
        xmin, ymin, xmax, ymax = [lon_bounds.min(), lat_bounds.min(), lon_bounds.max(), lat_bounds.max()]
        xres = (xmax - xmin) / float(nx)
        yres = (ymax - ymin) / float(ny)
        geotransform = np.round((xmin, xres, 0, 90 - yres * latpixels2skip, 0, -yres), 6)
        # print(geotransform)

        output_file = os.path.join(output_dir, tif_name)
        #first create the tiff in memory to add overviews in the second step. Last step is to copy it to a real file. This flow should be used to make a valid COG
        dst_ds = gdal.GetDriverByName('MEM').Create('', nx, ny - 2 * latpixels2skip, 1, dtype)
        dst_ds.SetGeoTransform(geotransform)  # specify coords
        dst_ds.SetMetadata(TIFTAGS)
        srs = osr.SpatialReference()  # establish encoding
        srs.ImportFromEPSG(4326)  # WGS84 lat/long
        dst_ds.SetProjection(srs.ExportToWkt())  # export coords to file
        out_band = dst_ds.GetRasterBand(1)
        #out_band.SetMetadata(metadata)
        out_band.SetNoDataValue(nodata)
        out_band.SetScale(scale)
        out_band.SetOffset(offset)
        out_band.SetUnitType(metadata['units'])
        out_band.SetDescription(metadata['description'])

        #only use data from lat 89 to -89
        #skip 40 pixel on top and on bottom
        gdalnumeric.BandWriteArray(out_band, d_var_scaled_masked[latpixels2skip:-latpixels2skip, :])
        dst_ds.BuildOverviews("NEAREST", [2, 4, 8, 16, 32])

        driver = gdal.GetDriverByName('GTiff')
        data_cog = driver.CreateCopy(output_file, dst_ds, options = ["COPY_SRC_OVERVIEWS=YES",
                                                                      'COMPRESS=DEFLATE',
                                                                      'ZLEVEL=9',
                                                                      'TILED=YES'])
        dst_ds.FlushCache()  # write to disk
        dst_ds = None
        data_cog= None