import argparse
import json
import os
import subprocess
import sys

from pyspark import SparkContext
from pyspark import SparkConf
print (os.environ['PYTHONPATH'])
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '.')))
ppath= os.environ['PYTHONPATH']

os.environ['PYTHONPATH'] = ppath + ':' + os.path.abspath(os.path.join(os.path.dirname(__file__), '.'))
#os.environ['PYSPARK_PYTHON'] = '/usr/bin/python3.5'
print (os.environ['PYTHONPATH'])
from s5pbira.s5p_product import s5p_out_product
from s5pbira.S5p_convert_nc_gtiff import convert_nc2tiff, get_footprint_value

import logging


pyspark_log = logging.getLogger('pyspark').setLevel(logging.ERROR)  # remove anoying pyspark messages
logger = logging.getLogger("py4j")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler())

def parse_arguments(argv):
    parser = argparse.ArgumentParser(prog='workflow_runner.py')

    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Increase output verbosity.')
    parser.add_argument('-c', '--config', metavar='FILE', required=False,
                        help='The workflow configuration file. Use simple relative path to indicate a file in the source e.g. "config_V200.ini"')
    parser.add_argument('--tmp_dir', metavar='DIRECTORY', default=os.getcwd(),
                        help='The temporary directory. Default is the current working directory.')
    parser.add_argument('--out_dir', metavar='DIRECTORY', default=None,
                        help='The output directory, will overwrite output dirs in config file. Default is None.')
    parser.add_argument('--log_file', metavar='FILE', default=None,
                        help='The logfile. Default is no logging to file.')
    parser.add_argument('--delete_tmp', action='store_true',
                        help='Remove temporary files after processing.')
    parser.add_argument('--overwrite', action='store_true',
                        help='overwrite files if they already exists.')
    parser.add_argument("--producttype", metavar='CO,HCHO,...', default=None,
                        help='the chemical substance')
    parser.add_argument("--productlevel", action='store_true',
                        help='L2 or L3 generation')
    parser.add_argument('-i', '--input', required=True,
                        help="input json file with run info")
    parser.add_argument('-t', "--composit_type", required=True,
                        help='TD|TM|TY')
    parser.add_argument('-s', '--simulation', action='store_true',
                        help='run in simulation mode')
    parser.add_argument('-V','--version', required=True,
                        help='Main product version 100|200|...')
    return parser.parse_known_args(argv)

def process(job, logger, simulation=False, overwrite=False):
    set_umask()

    logger.error('overwrite %s' % overwrite)
    cmd = '/opt/harp/bin/harpmerge ' + job.harp_parameters
    print(cmd)
    logger.info(cmd)

    out_nc_exist = os.path.exists(os.path.join(job.outfoldernc,job.outncfile))
    if simulation == False:

        #create netcdf file using harp
        if not out_nc_exist or (out_nc_exist and overwrite):
            logger.info ("Starting harpmerge command...")
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
            stdout, stderr = p.communicate()
            errNr = p.returncode
            if errNr > 0:
                logger.error ('************************** RESULT ************************')
                logger.error('errNr  : %s' % errNr)
                logger.error("stdout : %s" % stdout)
                logger.error("stderr : %s" % stderr)
                logger.error("***********************************************************")
                raise Exception("Error while executing harpmerge %s %s %s " %(errNr,stdout, stderr) )

        #create tiff files
        out_tiff_exist = os.path.exists(os.path.join(job.outfoldertif, job.outncfile.replace(".nc",'.tif')))
        if not out_tiff_exist or (out_tiff_exist and overwrite):
            logger.info("Converting NC to GeoTiff: %s " % job.outncfile)
            nc_file = os.path.join(job.outfoldernc, job.outncfile)
            convert_nc2tiff(nc_file, job.outfoldertif, job.tif_settings)
            logger.info("DONE converting NC to GeoTiff: %s " % job.outncfile)
    else:
        logger.info("this is a simulation run : ")
    return job

def set_umask():
    """Set umask for right permissions"""
    try:
        os.umask(0o000)
    except:
        pass

def main(argv):

    args, unknown_args = parse_arguments(argv)
    #open input json file for parsing
    with open(args.input) as f:
        in_joblst = json.load(f)

    #remove the output json file before the run
    out_json = args.input + '_out.json'
    if os.path.exists(out_json):
        os.remove(out_json)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)
    sparkApplicationId = sc.applicationId;



    logger.info("spark id : %s" % sparkApplicationId)

    #loop over all possible jobs(days)
    try:

        # prepare jobs, set the metadata
        out_joblst = []
        for job in in_joblst:
            s5p_prod = s5p_out_product(job, logger, args.version, args.composit_type)
            s5p_prod.update_metadata_json()
            out_joblst.append(s5p_prod)


        #execute jobs in parallel
        numberOfJobs = len(out_joblst)
        if numberOfJobs > 0:
            jobs_rdd = sc.parallelize(out_joblst, len(out_joblst))
            jobs_rdd.foreach(lambda job: process(job, logger, args.simulation))

        # calcuate simple footprint and put this in the job.content dictionary
        json_content = list(map(lambda job: job.set_metadata("footprint", get_footprint_value(
            os.path.join(job.content['outdir_tif'],
                         job.content['output_tif_list'][0]))), out_joblst))

        #write all output to a json file
        json_content = list(map(lambda job : job.content, out_joblst))
        with open(out_json, 'w') as f:
            json.dump(json_content, f, indent=4)
        logger.info("writing output json %s" % out_json)
    finally:
        sc.stop()


if __name__ == '__main__':
    main(sys.argv)